import { RecordQueryBuilder, InitializedRecord, RecordQueryExpressionResult } from '@orbit/records';
import { QueryKey, QueryClientConfig, QueryClient, InfiniteQueryObserver, InfiniteQueryObserverOptions, QueryObserver, QueryObserverOptions } from '@tanstack/query-core';
export { QueryMeta } from '@tanstack/query-core';
import { MemorySource } from '@orbit/memory';

declare type GetQueryOrExpressions<TQueryKey extends QueryKey = QueryKey> = (queryBuilder: RecordQueryBuilder, queryKey: TQueryKey, pageParam?: number) => Parameters<MemorySource["query"]>[0];
declare module "@tanstack/query-core" {
    interface QueryMeta {
        getQueryOrExpressions?: GetQueryOrExpressions;
        isInfinite?: boolean;
        pageSize?: number;
    }
}

interface LiveQueryClientConfig extends QueryClientConfig {
    memorySource: MemorySource;
}
declare class LiveQueryClient extends QueryClient {
    private readonly memorySource;
    private readonly liveQueryAdapterCache;
    private memorySourceIsActivated;
    constructor(config: LiveQueryClientConfig);
    getMemorySource(): MemorySource;
    getMemorySourceIfActivated(): MemorySource | undefined;
}

declare class LiveInfiniteQueryObserver<TQueryFnData extends InitializedRecord[] = InitializedRecord[], TError = unknown, TData = TQueryFnData, TQueryData = TQueryFnData, TQueryKey extends QueryKey = QueryKey> extends InfiniteQueryObserver<TQueryFnData, TError, TData, TQueryData, TQueryKey> {
    constructor(client: LiveQueryClient, options: InfiniteQueryObserverOptions<TQueryFnData, TError, TData, TQueryData, TQueryKey>);
    setOptions(options?: Parameters<InfiniteQueryObserver<TQueryFnData, TError, TData, TQueryData, TQueryKey>["setOptions"]>[0], notifyOptions?: Parameters<InfiniteQueryObserver<TQueryFnData, TError, TData, TQueryData, TQueryKey>["setOptions"]>[1]): void;
}

declare class LiveQueryObserver<TQueryFnData extends Exclude<RecordQueryExpressionResult, null> = Exclude<RecordQueryExpressionResult, null>, TError = unknown, TData = TQueryFnData, TQueryData = TQueryFnData, TQueryKey extends QueryKey = QueryKey> extends QueryObserver<TQueryFnData, TError, TData, TQueryData, TQueryKey> {
    constructor(client: LiveQueryClient, options: QueryObserverOptions<TQueryFnData, TError, TData, TQueryData, TQueryKey>);
}

export { GetQueryOrExpressions, LiveInfiniteQueryObserver, LiveQueryClient, LiveQueryClientConfig, LiveQueryObserver };
